"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (_ref) {
    var t = _ref.types;

    var refAttributeBuilder = {
        asThisProperty: function asThisProperty(elementName, options) {
            var pathSource = createPathSourceStartsWithThis(options.path);
            var path = createPath(pathSource);

            return createRefExpression(elementName, path);
        },
        legacy: function legacy(elementName) {
            return t.stringLiteral(elementName);
        }
    };

    function createPathSourceStartsWithThis(source) {
        source = trimString(source);

        if (!source) return "this";

        if (source.startsWith("this")) return source;

        if (source.startsWith("[")) return "this" + source;

        return "this." + source;
    }

    function createPath(pathString) {
        return trimString(pathString).split(pathSeparator).map(trimString).filter(hasValue);
    }

    function createRefExpression(name, path) {
        path.push(name);

        var params = [t.identifier(name)];
        var body = t.assignmentExpression("=", buildPathAST(path), t.identifier(name));
        var arrowFunction = t.arrowFunctionExpression(params, body);

        return t.jSXExpressionContainer(arrowFunction);
    }

    function buildPathAST(path) {
        return path.map(pathTokenToNode).filter(hasValue).reduce(nodeToExpression);
    }

    function pathTokenToNode(token) {
        if (token == "this") return t.thisExpression();

        if (identifierTester.test(token)) return t.identifier(token);

        if (numericLiteralTester.test(token)) return t.numericLiteral(+token);

        if (stringLiteralTester.test(token)) return t.stringLiteral(literalToString(token));

        return null;
    }

    function nodeToExpression(expression, node) {
        var computed = t.isNumericLiteral(node) || t.isStringLiteral(node);

        return t.memberExpression(expression, node, computed);
    }

    function appendRefAttribute(path, elementName, options) {
        var refAttributeBuilderType = options.refType || defaultRefAttributeBuilderType;

        if (typeof refAttributeBuilder[refAttributeBuilderType] != "function") {
            console.error(errors.UNKNOWN_REFTYPE);
            refAttributeBuilderType = defaultRefAttributeBuilderType;
        }

        var attributeName = t.jSXIdentifier("ref");
        var attributeValue = refAttributeBuilder[refAttributeBuilderType](elementName, options);

        var attribute = t.jSXAttribute(attributeName, attributeValue);

        path.unshiftContainer("attributes", attribute);
    }

    return {
        inherits: require("babel-plugin-syntax-jsx"),

        visitor: {
            JSXNamespacedName: function JSXNamespacedName(path, state) {
                var namespaceNode = path.get("namespace").node;
                var elementName = namespaceNode.name;
                var jsxOpeningElement = path.findParent(function (path) {
                    return path.isJSXOpeningElement();
                });

                appendRefAttribute(jsxOpeningElement, elementName, state.opts);

                var tagNode = path.get("name").node;
                var tagName = tagNode.name;

                path.replaceWith(t.jSXIdentifier(tagName));
            }
        }
    };
};

var defaultRefAttributeBuilderType = "asThisProperty";
var pathSeparator = /[.\[\]]/;
var identifierTester = /^[A-z$][\w$]*$/;
var stringLiteralTester = /^(["].*["])|(['].*['])$/;
var numericLiteralTester = /^\d+$/;
var errors = {
    UNKNOWN_REFTYPE: "unknown refType, using default"
};

function hasValue(value) {
    return value;
}

function literalToString(stringLiteral) {
    return stringLiteral.slice(1, -1);
}

function trimString(string) {
    return typeof string == "string" ? string.trim() : "";
}

;